@@ -376,6 +376,7 @@ uint32_t gssntlm_acquire_cred_from(uint32_t *minor_status,
         }
         if (retmin) {
             set_GSSERR(retmin);
+            goto done;
         }
     } else if (cred_usage == GSS_C_ACCEPT) {
         if (name != NULL && name->type != GSSNTLM_NAME_SERVER) {
@@ -386,11 +387,14 @@ uint32_t gssntlm_acquire_cred_from(uint32_t *minor_status,
         retmin = get_server_creds(name, cred);
         if (retmin) {
             set_GSSERR(retmin);
+            goto done;
         }
     } else if (cred_usage == GSS_C_BOTH) {
         set_GSSERRS(ERR_NOTSUPPORTED, GSS_S_CRED_UNAVAIL);
+        goto done;
     } else {
         set_GSSERRS(ERR_BADARG, GSS_S_CRED_UNAVAIL);
+        goto done;
     }
 
     set_GSSERRS(0, GSS_S_COMPLETE);
@@ -477,17 +481,25 @@ uint32_t gssntlm_inquire_cred(uint32_t *minor_status,
                               gss_cred_usage_t *cred_usage,
                               gss_OID_set *mechanisms)
 {
-    struct gssntlm_cred *cred;
+    struct gssntlm_cred *cred = (struct gssntlm_cred *)GSS_C_NO_CREDENTIAL;
     uint32_t retmin, retmaj;
     uint32_t maj, min;
 
     if (cred_handle == GSS_C_NO_CREDENTIAL) {
-        set_GSSERRS(ERR_NOARG, GSS_S_NO_CRED);
-        goto done;
+        maj = gssntlm_acquire_cred_from(&min,
+                                        NULL, GSS_C_INDEFINITE,
+                                        NULL, GSS_C_INITIATE,
+                                        GSS_C_NO_CRED_STORE,
+                                        (gss_cred_id_t *)&cred,
+                                        NULL, NULL);
+        if (maj) {
+            set_GSSERRS(0, GSS_S_NO_CRED);
+            goto done;
+        }
+    } else {
+        cred = (struct gssntlm_cred *)cred_handle;
     }
 
-    cred = (struct gssntlm_cred *)cred_handle;
-
     if (cred->type == GSSNTLM_CRED_NONE) {
         set_GSSERRS(ERR_BADARG, GSS_S_NO_CRED);
         goto done;
@@ -559,6 +571,9 @@ uint32_t gssntlm_inquire_cred(uint32_t *minor_status,
     set_GSSERRS(0, GSS_S_COMPLETE);
 
 done:
+    if (cred_handle == GSS_C_NO_CREDENTIAL) {
+        gssntlm_release_cred(&min, (gss_cred_id_t *)&cred);
+    }
     return GSSERR();
 }
 
