@@ -430,6 +430,7 @@ done:
         (retmaj != GSS_S_CONTINUE_NEEDED)) {
         gssntlm_delete_sec_context(&tmpmin, (gss_ctx_id_t *)&ctx, NULL);
     } else {
+        if (actual_mech_type) *actual_mech_type = discard_const(&gssntlm_oid);
         if (ret_flags) *ret_flags = ctx->gss_flags;
         if (time_rec) *time_rec = GSS_C_INDEFINITE;
     }
@@ -990,6 +991,7 @@ done:
         (retmaj != GSS_S_CONTINUE_NEEDED)) {
         gssntlm_delete_sec_context(&tmpmin, (gss_ctx_id_t *)&ctx, NULL);
     } else {
+        if (mech_type) *mech_type = discard_const(&gssntlm_oid);
         if (ret_flags) *ret_flags = ctx->gss_flags;
         if (time_rec) *time_rec = GSS_C_INDEFINITE;
     }
@@ -1091,6 +1093,64 @@ gss_OID_desc set_seq_num_oid = {
     discard_const(GSS_NTLMSSP_SET_SEQ_NUM_OID_STRING)
 };
 
+uint32_t gssntlm_set_seq_num(uint32_t *minor_status,
+                             struct gssntlm_ctx *ctx,
+                             const gss_buffer_t value)
+{
+    uint32_t retmin;
+    uint32_t retmaj;
+
+    if (ctx->gss_flags & GSS_C_DATAGRAM_FLAG) {
+        if (value->length != 4) {
+            return GSSERRS(ERR_BADARG, GSS_S_FAILURE);
+        }
+        memcpy(&ctx->crypto_state.recv.seq_num,
+               value->value, value->length);
+        ctx->crypto_state.send.seq_num = ctx->crypto_state.recv.seq_num;
+    } else {
+        return GSSERRS(ERR_WRONGCTX, GSS_S_FAILURE);
+    }
+
+    return GSSERRS(0, GSS_S_COMPLETE);
+}
+
+gss_OID_desc reset_crypto_oid = {
+    GSS_NTLMSSP_RESET_CRYPTO_OID_LENGTH,
+    discard_const(GSS_NTLMSSP_RESET_CRYPTO_OID_STRING)
+};
+
+uint32_t gssntlm_reset_crypto(uint32_t *minor_status,
+                              struct gssntlm_ctx *ctx,
+                              const gss_buffer_t value)
+{
+    uint32_t retmin;
+    uint32_t retmaj;
+
+    if (value->length != 4) {
+        return GSSERRS(ERR_BADARG, GSS_S_FAILURE);
+    }
+
+    /* reset crypto state */
+    if (ctx->neg_flags & (NTLMSSP_NEGOTIATE_SIGN |
+                            NTLMSSP_NEGOTIATE_SEAL)) {
+        uint32_t val;
+
+        memcpy(&val, value->value, value->length);
+
+        /* A val of 1 means we want to reset the verifier handle,
+         * which is the receive handle for NTLM, otherwise we reset
+         * the send handle. */
+        retmin = ntlm_reset_rc4_state(ctx->neg_flags, (val == 1),
+                                      &ctx->exported_session_key,
+                                      &ctx->crypto_state);
+        if (retmin) {
+            return GSSERRS(retmin, GSS_S_FAILURE);
+        }
+    }
+
+    return GSSERRS(0, GSS_S_COMPLETE);
+}
+
 uint32_t gssntlm_set_sec_context_option(uint32_t *minor_status,
                                         gss_ctx_id_t *context_handle,
                                         const gss_OID desired_object,
@@ -1111,28 +1171,12 @@ uint32_t gssntlm_set_sec_context_option(uint32_t *minor_status,
 
     /* set seq num */
     if (gss_oid_equal(desired_object, &set_seq_num_oid)) {
-        if (ctx->gss_flags & GSS_C_DATAGRAM_FLAG) {
-
-            if (value->length != 4) {
-                set_GSSERR(ERR_BADARG);
-                goto done;
-            }
-
-            memcpy(&ctx->crypto_state.recv.seq_num,
-                   value->value, value->length);
-            ctx->crypto_state.send.seq_num = ctx->crypto_state.recv.seq_num;
-            set_GSSERRS(0, GSS_S_COMPLETE);
-            goto done;
-        } else {
-            set_GSSERRS(ERR_WRONGCTX, GSS_S_FAILURE);
-            goto done;
-        }
+        return gssntlm_set_seq_num(minor_status, ctx, value);
+    } else if (gss_oid_equal(desired_object, &reset_crypto_oid)) {
+        return gssntlm_reset_crypto(minor_status, ctx, value);
     }
 
-    set_GSSERRS(ERR_BADARG, GSS_S_UNAVAILABLE);
-
-done:
-    return GSSERR();
+    return GSSERRS(ERR_BADARG, GSS_S_UNAVAILABLE);
 }
 
 gss_OID_desc spnego_req_mic_oid = {
@@ -1140,29 +1184,16 @@ gss_OID_desc spnego_req_mic_oid = {
     discard_const(GSS_SPNEGO_REQUIRE_MIC_OID_STRING)
 };
 
-uint32_t gssntlm_inquire_sec_context_by_oid(uint32_t *minor_status,
-	                                    const gss_ctx_id_t context_handle,
-	                                    const gss_OID desired_object,
-	                                    gss_buffer_set_t *data_set)
+uint32_t gssntlm_spnego_req_mic(uint32_t *minor_status,
+                                struct gssntlm_ctx *ctx,
+                                gss_buffer_set_t *data_set)
 {
-    struct gssntlm_ctx *ctx;
     gss_buffer_desc mic_buf;
-    uint32_t retmaj, retmin, tmpmin;
+    uint32_t retmin;
+    uint32_t retmaj;
+    uint32_t tmpmin;
     uint8_t mic_set;
 
-    if (context_handle == NULL) {
-        return GSSERRS(ERR_NOARG, GSS_S_CALL_INACCESSIBLE_READ);
-    }
-    if (desired_object == GSS_C_NO_OID) {
-        return GSSERRS(ERR_NOARG, GSS_S_CALL_INACCESSIBLE_READ);
-    }
-    if (!data_set) {
-        return GSSERRS(ERR_NOARG, GSS_S_CALL_INACCESSIBLE_WRITE);
-    }
-
-    ctx = (struct gssntlm_ctx *)context_handle;
-    *data_set = GSS_C_NO_BUFFER_SET;
-
     /* the simple fact the spnego layer is asking means it can handle
      * forcing mechlistMIC if we add a MIC to the Authenticate packet.
      * We expect this to be called before the authenticate token is
@@ -1187,3 +1218,33 @@ uint32_t gssntlm_inquire_sec_context_by_oid(uint32_t *minor_status,
 
     return GSSERRS(retmin, retmaj);
 }
+
+
+uint32_t gssntlm_inquire_sec_context_by_oid(uint32_t *minor_status,
+	                                    const gss_ctx_id_t context_handle,
+	                                    const gss_OID desired_object,
+	                                    gss_buffer_set_t *data_set)
+{
+    struct gssntlm_ctx *ctx;
+    uint32_t retmin;
+    uint32_t retmaj;
+
+    if (context_handle == NULL) {
+        return GSSERRS(ERR_NOARG, GSS_S_CALL_INACCESSIBLE_READ);
+    }
+    if (desired_object == GSS_C_NO_OID) {
+        return GSSERRS(ERR_NOARG, GSS_S_CALL_INACCESSIBLE_READ);
+    }
+    if (!data_set) {
+        return GSSERRS(ERR_NOARG, GSS_S_CALL_INACCESSIBLE_WRITE);
+    }
+
+    ctx = (struct gssntlm_ctx *)context_handle;
+    *data_set = GSS_C_NO_BUFFER_SET;
+
+    if (gss_oid_equal(desired_object, &spnego_req_mic_oid)) {
+        return gssntlm_spnego_req_mic(minor_status, ctx, data_set);
+    }
+
+    return GSSERRS(ERR_NOTSUPPORTED, GSS_S_UNAVAILABLE);
+}
